from PIL import Image
from PIL import ImageFont
from PIL import ImageDraw

img = Image.open("image.jpg")
draw = ImageDraw.Draw(img)
font = ImageFont.truetype("impact.ttf", 42)

def drawTextWithOutline(text, x, y):
    draw.text((x-2, y-2), text,(0,0,0),font=font)
    draw.text((x+2, y-2), text,(0,0,0),font=font)
    draw.text((x+2, y+2), text,(0,0,0),font=font)
    draw.text((x-2, y+2), text,(0,0,0),font=font)
    draw.text((x, y), text, (255,255,255), font=font)
    return


def drawText(text, pos):
    text = text.upper()
    w, h = draw.textsize(text, font) # measure the size the text will take

    lineCount = 1
    if w > img.width:
        lineCount = int(round((w / img.width) + 1))

    # print("lineCount: {}".format(lineCount))

    lines = []
    if lineCount > 1:

        lastCut = 0
        isLast = False
        for i in range(0,lineCount):
            if lastCut == 0:
                cut = (len(text) / lineCount) * i
            else:
                cut = lastCut

            if i < lineCount-1:
                nextCut = (len(text) / lineCount) * (i+1)
            else:
                nextCut = len(text)
                isLast = True

            # print("cut: {} -> {}".format(cut, nextCut))

            # make sure we don't cut words in half
            if nextCut == len(text) or text[nextCut] == " ":
                # print("may cut")
            else:
                # print("may not cut")
                while text[nextCut] != " ":
                    nextCut += 1
                # print("new cut: {}".format(nextCut))

            line = text[cut:nextCut].strip()

            # is line still fitting ?
            w, h = draw.textsize(line, font)
            if not isLast and w > img.width:
                # print("overshot")
                nextCut -= 1
                while text[nextCut] != " ":
                    nextCut -= 1
                # print("new cut: {}".format(nextCut))

            lastCut = nextCut
            lines.append(text[cut:nextCut].strip())

    else:
        lines.append(text)

    # print(lines)

    lastY = -h
    if pos == "bottom":
        lastY = img.height - h * (lineCount+1) - 10

    for i in range(0, lineCount):
        w, h = draw.textsize(lines[i], font)
        x = img.width/2 - w/2
        y = lastY + h
        drawTextWithOutline(lines[i], x, y)
        lastY = y


drawText("One does not simply", "top")
drawText("make memes great again with several lines", "bottom")

img.save("out.jpg")



from PIL import Image
from PIL import ImageFont
from PIL import ImageDraw

img = Image.open("image.jpg")
draw = ImageDraw.Draw(img)
font = ImageFont.truetype("impact.ttf", 42)

def drawTextWithOutline(text, x, y):
    draw.text((x-2, y-2), text,(0,0,0),font=font)
    draw.text((x+2, y-2), text,(0,0,0),font=font)
    draw.text((x+2, y+2), text,(0,0,0),font=font)
    draw.text((x-2, y+2), text,(0,0,0),font=font)
    draw.text((x, y), text, (255,255,255), font=font)
    return

text = "Are we centered yet?"
w, h = draw.textsize(text, font) # measure the size the text will take
drawTextWithOutline(text, img.width/2 - w/2, 10)

img.save("out.jpg")



#from LangDeckGen import Config
from LangDeckGen import pixabay as pb
import errno
import json
import logging
import mimetypes
import os
import shutil
import sys
from pathlib import Path
import requests
from PIL import Image
from io import BytesIO
from bs4 import BeautifulSoup
from slugify import slugify
import time
from googletrans import Translator
from gtts import gTTS
import LangDeckGen
from LangDeckGen.memecaption import meme
from LangDeckGen.audiospeed import audiospeed
from pathlib import Path
import re
from bing_image_downloader import downloader
import validators
from pathlib import Path
language_code={'fr':'french','de':'german','sv':'swedish','fi':'finnish','es':'spanish'}
WAIT_TIME=2


def suppressOutput(func):
    def wrapper(*args, **kwargs):
        with open(os.devnull,"w") as devNull:
            original = sys.stdout
            sys.stdout = devNull
            func(*args, **kwargs)
            sys.stdout = original
    return wrapper

special_characters = ['!','#','$','%', '&','@','[',']',']','_',"'",'"',',']

downloader.download = suppressOutput(downloader.download)


class AnkiCard:

    def __init__(self, word: str, lang: str, *args,**kwargs):
        self.PIXABAY_KEY = os.environ.get("PIXABAY_KEY")
        self.OUTPUT_DIRECTORY = Path("./tmp").absolute()
        self.word = word
        self.lang = lang
        self.cardID=self.__create_id() 
        self.translation_dict = self.__translate_word(word,**kwargs)
        self.audio = self.__voice_translate(self.translation_dict["tl"],**kwargs)
        if kwargs.get("PhrasesOnly"):
            self.sentence = ["","","",""]
        else:
            self.sentence = self.__get_example_sentence()
        try:
            self.image = self.__download_word_image(*args,**kwargs)
        except:
            self.image = Path(LangDeckGen.__file__).parent / 'templates' / 'placeholder.png' 
            self.image = str(self.image)
        if not kwargs.get("PhrasesOnly"):
            self.sentence_audio = self.__generate_sentence_audio(**kwargs)
            self.dictionary_entry = self.__get_dictionary_entry()
    def __str__(self):
        return f"TLWord:{self.word}\n ENword:{self.translation_dict['en']}\n AUDIO_FILE:{self.audio}\n \
                 IMAGE_FILE: {self.image} SENTENCES: {self.sentence} SENTENCES_AUDIO_FILES:{self.sentence_audio}"

    def __create_id(self):
        id=0
        for char in self.word+self.lang:
            id+=ord(char)
        return id

    def __translate_word(self, word,**kwargs):
        translation_dict = {"en": "", "tl": ""}
        translator = Translator()
        translation_dict["tl"]=word
        time.sleep(WAIT_TIME)
        print('word',word)
        translation_dict["en"]=translator.translate(word, src=self.lang, dest='en').text
        if kwargs.get('back',False):    ## override tranlation with what was given
            translation_dict["en"]=kwargs['back']

        

        return translation_dict


    def __generate_sentence_audio(self,**kwargs):
        sentence_audio = list()
        TL_sentences = self.sentence[0::2]
        for sentence in TL_sentences:
            sentence_audio.append(self.__voice_translate(sentence,**kwargs))
        return sentence_audio

    def __voice_translate(self, phrase: str, **kwargs) -> str:
        # Define the path to tmp/sound
        audio_path = str(self.OUTPUT_DIRECTORY)+"/"+"sound"
        phraseprint = re.sub(r"[^a-zA-Z0-9 ]", "", phrase) ## remove special chars
        mp3_name = f"{phraseprint[:30].replace(' ','-')}{self.cardID}.mp3"
        # Join all the path ingredients together
        audio_file = audio_path+"/"+mp3_name

        time.sleep(WAIT_TIME) 
        if phrase == "":
            phrase = "okay"
        if kwargs.get('comment_symbol',False):
            phrase=phrase.split(kwargs['comment_symbol'])[0] ## ignore observations that follow after "- "
        tts = gTTS(phrase, lang=self.lang)

        tts.save(audio_file)
        if kwargs.get("tts_speed"):
            audio_file=audiospeed.ChangeAudioSpeed(audio_file,kwargs.get("tts_speed"))
        return(f"{audio_file}")

    def __download_word_image(self,*args,**kwargs):
        if not kwargs.get("UsePixabay"):
            newImgName=self.translation_dict["tl"][:30].replace(' ','-')+str(self.cardID)
            nimg=args[0] ## number of the image to take 1 to 3 
            img_dir= Path('./tmp/imgs').absolute()        
            

            if nimg.strip().isdigit():
                query_string=self.translation_dict["en"]
                gotFigure=True
                try:
                    downloader.download(query_string, limit=3,  output_dir='tmp', adult_filter_off=False, force_replace=False, timeout=60, verbose=True)  ## by default bing downloader will create a folder for every query inside output_dir
                except:
                    gotFigure=False

                for i in special_characters:
                    query_string = query_string.replace(i,'')
                bing_dir = Path('tmp').joinpath(query_string).absolute()
                try:
                    if not Path.is_dir(img_dir):
                        Path.mkdir(img_dir, parents=True)
                except:
                    print("couldnt create folder")
                try:
                    if not Path.is_dir(bing_dir):
                        Path.mkdir(bing_dir, parents=True)
                except:
                    print("couldnt create folder")

                if not gotFigure: ## case bing didnt bring any pics
                    imgformat='jpg'
                    newImgName=self.translation_dict["tl"][:30].replace(' ','-')+str(self.cardID)+'.'+imgformat
                    fullImgName='/'.join((str(img_dir),newImgName))
                    try:
                        if kwargs.get("PhrasesOnly"):
                            meme.DrawMeme(meme.getRandomBasePicture(),"",self.translation_dict["tl"],
                                      full_imgname)
                            meme.DrawMeme(meme.getRandomBasePicture(),self.translation_dict["tl"],self.sentence[0],
                                      full_imgname)
                    except:
                        print(f"Couldnt draw phrase in figure for {self.translation_dict['tl']}")
                    return full_imgname
                else:
                    listing=os.listdir(bing_dir)
                    imgformat='jpg'
                    try:
                        for each in listing:
                            if f"Image_{nimg}" in each:
                                imgformat=each.split('.')[-1]
                    except:
                        print('Didnt find extension, using jpg.')
                    
                    newImgName=newImgName+'.'+imgformat
                    shutil.copy(str(bing_dir.joinpath(f"Image_{nimg}.{imgformat}")), str(img_dir.joinpath(f"{newImgName}")))
                    shutil.rmtree(bing_dir)
                    fullImgName='/'.join((str(img_dir),newImgName))
                    try:
                        if kwargs.get("PhrasesOnly"):
                            meme.DrawMeme(fullImgName,"",self.translation_dict["tl"],fullImgName)
                        else:
                            meme.DrawMeme(fullImgName,self.translation_dict["tl"],self.sentence[0],
                                      fullImgName)
                    except:
                        print(f"Couldnt draw phrase in figure for {self.translation_dict['tl']}")
                    return fullImgName
            elif validators.url(nimg):  ## casearg is an url
                response = requests.get(nimg.strip())
                #ext=nimg.strip().split(".")[-1] ## get extension
                img = Image.open(BytesIO(response.content))
                try:
                    imgname=newImgName+".jpg"
                    fullImgName='/'.join((str(img_dir),imgname))
                    img.save(fullImgName)
                except:
                    imgname=newImgName+".png"
                    fullImgName='/'.join((str(img_dir),imgname))
                    img.save(fullImgName)
                return fullImgName

            else:  ## case arg is path 
                local_path= Path('.').absolute()        
                fullImgName = '/'.join((str(local_path),str(nimg)))
                return fullImgName
            
        else:
            bay = pb.PixaBay(key=self.PIXABAY_KEY)
            time.sleep(WAIT_TIME) 
            imgs = bay.get_images(search=self.translation_dict["en"])
            img_path = self.OUTPUT_DIRECTORY / "imgs"
            imgname=self.translation_dict["tl"][:30].replace(' ','-')+str(self.cardID)
            full_imgname=""
            if imgs.total != 0: 
                img_def=args[0]
                if img_def.strip().isdigit():
                    img=imgs.get_img(int(img_def.strip())-1) ## if specified arg can take another figure in the hits
                    imgname=img.download(img_path,name=imgname,size='web')
                    full_imgname='/'.join((str(img_path),imgname))
                else:  ## case arg is an url 
                    response = requests.get(img_def.strip())
                    ext=img_def.strip().split(".")[-1] ## get extension
                    img = Image.open(BytesIO(response.content))
                    try:
                        imgname=imgname+".jpg"
                        full_imgname='/'.join((str(img_path),imgname))
                        img.save(full_imgname)
                    except:
                        imgname=imgname+".png"
                        full_imgname='/'.join((str(img_path),imgname))
                        img.save(full_imgname)

                if not self.sentence==["","","","",""]: ## case of personal pronouns and maybe others
                    print(full_imgname,self.sentence[0],self.translation_dict["tl"])
                    meme.DrawMeme(full_imgname,self.translation_dict["tl"],self.sentence[0],
                              full_imgname)
                return full_imgname
            else: ## if no image is found in pixabay for the word
                full_imgname=str(img_path)+"/"+self.translation_dict["tl"]+str(self.cardID)+'.jpg'
                if not self.sentence==["","","","",""]: ## case of personal pronouns and maybe others
                    meme.DrawMeme(meme.getRandomBasePicture(),self.translation_dict["tl"],self.sentence[0],
                              full_imgname)
                return full_imgname

    def __get_example_sentence(self):
        language_ext=language_code[self.lang]
        req = requests.get(f"https://context.reverso.net/translation/{language_ext}-english/{self.translation_dict['tl']}",
                           headers={'User-Agent': 'Mozilla/5.0'})
        time.sleep(WAIT_TIME)
        soup = BeautifulSoup(req.text, 'lxml')
        sentences = [x.text.strip() for x in soup.find_all('span', {'class': 'text'}) if '\n' in x.text]
        if len(sentences) == 0 : # empty
            sentences=["-","-","-","-","-"]
        sentences.extend(["-","-","-","-","-"]) ## to be on safe side to have sentences filled.
        return sentences[:4]

    def __get_dictionary_entry(self):
        language_ext=language_code[self.lang]
        req = requests.get(f"https://dictionary.reverso.net/{language_ext}-english/{self.translation_dict['tl']}",
                   headers={'User-Agent': 'Mozilla/5.0'})
        soup = BeautifulSoup(req.text, 'html.parser')
        html_dict_entry=soup.find('div', {'id':"TableHTMLResult"})
        ## to remove some bad formatted icons  ## but it doesnt hide everything anymore now, try to fix later
#        html_dict_entry = re.sub("\<!-- google_ad_section_end --\>.*\<!-- end Block with commentaries --\>", '', str(html_dict_entry),0,re.DOTALL)
#        html_dict_entry="<div>\n"+html_dict_entry+"\n</div>"
#        html_dict_entry= re.sub("</div>\s*$","",str(html_dict_entry),0,re.DOTALL)
        return str(html_dict_entry)






import os

import uuid
from pathlib import Path


def Config():
    # Azure Config
#    AZURE_SPEECH_KEY = os.environ.get("AZ-SPEECH-KEY")
#    AZURE_TRANSLATE_KEY = os.environ.get("AZ-TRANS-KEY")
#    AZURE_SSML_CONF = Path("ssml.xml")
    PIXABAY_KEY = os.environ.get("PIXABAY_KEY")

    # Google Config
#    GOOGLE_TRANSLATE_KEY = os.environ.get("GOOGLE-TRANS-KEY")

#    TRANSLATE_API_KEY = (GOOGLE_TRANSLATE_KEY if AZURE_TRANSLATE_KEY is None else AZURE_TRANSLATE_KEY)

#    VOICE_SUBSCRIPTION_REGION = (
#        os.environ.get("AZ-SUB-REGION") if os.environ.get("AZ-SUB-REGION") is not None else "australiaeast"
#    )

#    TRANSLATE_SUBSCRIPTION_REGION = (
#        os.environ.get("AZ-TRANS-REGION") if os.environ.get("AZ-TRANS-REGION") is not None else "australiaeast"
#    )

#    AZURE_IMAGE_API_URL = "https://api.bing.microsoft.com/v7.0/images/search"
#    AZURE_TRANSLATE_API_URL = "https://api.cognitive.microsofttranslator.com/translate?api-version=3.0&from={}&to={}"

#    GOOGLE_IMAGE_API_URL = ""
#    GOOGLE_TRANSLATE_API_URL = "https://translation.googleapis.com/language/translate/v2?format=text&source={}&target={}&key="+GOOGLE_TRANSLATE_KEY

#    TRANSLATE_API_URL = (GOOGLE_TRANSLATE_API_URL if AZURE_TRANSLATE_KEY is None else AZURE_TRANSLATE_API_URL)

#    ACCEPTED_IMG_FORMATS = ['jpeg', 'jpg', 'png', 'gif']

#    AZURE_HEADERS = {
#        'Ocp-Apim-Subscription-Key': AZURE_TRANSLATE_KEY,
#        'Content-type': 'application/json',
#        'X-ClientTraceId': str(uuid.uuid4()),
#        'Ocp-Apim-Subscription-Region': TRANSLATE_SUBSCRIPTION_REGION
#    }

#    BING_HEADERS = {"Ocp-Apim-Subscription-Key": f"{BING_SEARCH_KEY}"}
#    BING_PARAMS = {
#        "mkt": "de-DE",
#        "imageType": "Photo",
#        "count": "3"
#    }



from typing import List

import genanki
from LangDeckGen.AnkiCard import AnkiCard


class AnkiDeck(genanki.Deck):
    deck_id = str
    anki_cards = List[AnkiCard]
    model = genanki.Model

    def __init__(self, title: str, anki_cards: List[AnkiCard], **kwargs):
        super().__init__(name=title, deck_id=hash(title))
        self.anki_cards = anki_cards

    def __build_media_lib(self,**kwargs):
        media = list()
        for card in self.anki_cards:
            media.append(card.image)
            media.append(card.audio)
            if not kwargs.get("PhrasesOnly"):
                media.extend(card.sentence_audio)
        return media

    def package_deck(self, anki_deck: genanki.Deck, **kwargs) -> genanki.Package:
        package = genanki.Package(anki_deck)
        package.media_files = self.__build_media_lib(**kwargs)
        return package

    def create_notes(self, model: genanki.Model,**kwargs):
        notes = list()
        for card in self.anki_cards:
            if not kwargs.get("PhrasesOnly"):
                notes.append(genanki.Note(
                    model=model,
                    fields=[card.translation_dict["tl"],
                            card.translation_dict["en"],
                            f"[sound:{card.audio.split('/')[-1]}]",
                            card.sentence[0],
                            card.sentence[1],
                            f"[sound:{card.sentence_audio[0].split('/')[-1]}]",
                            card.sentence[2],
                            card.sentence[3],
                            f"[sound:{card.sentence_audio[1].split('/')[-1]}]",
            #                card.sentence[5],
            #                card.sentence[4],
            #                f"[sound:{card.sentence_audio[2]}]",
            #                card.sentence[7],
            #                card.sentence[6],
            #                f"[sound:{card.sentence_audio[3]}]",
                            f"<img src='{card.image.split('/')[-1]}'>",
                            card.dictionary_entry]
                    ))
            else:
                notes.append(genanki.Note(
                    model=model,
                    fields=[card.translation_dict["tl"],
                            card.translation_dict["en"],
                            f"[sound:{card.audio.split('/')[-1]}]",
                            ""]
                            # f"<img src='{card.image.split('/')[-1]}'>"]
                    ))

        return notes

    def __str__(self):
        return print(f"<AnkiDeck {self.name} {self.deck_id} {len(self.anki_cards)}>")



from PIL import Image
from PIL import ImageFont
from PIL import ImageDraw

def getRandomBasePicture():
    import os, random
    filefolder='/'.join(__file__.split('/')[:-1])
    base_img=random.choice(os.listdir(filefolder+'/images'))
    return filefolder+'/images/'+base_img

class DrawMeme:
    def __init__(self,base_img,textup,textdown,savefig):
        self.base_img=base_img
        self.textup=textup
        self.textdown=textdown
        self.savefig=savefig
        self.drawMeme()
    def drawMeme(self):
        img = Image.open(self.base_img)
        draw = ImageDraw.Draw(img)
        self.drawText(img, draw, self.textup.upper(), "top")
        self.drawText(img, draw, self.textdown.upper(), "bottom")
        img.save(self.savefig)

    def drawText(self, img,draw, msg, pos):
        fontSize = 56;
        lines = []
        filefolder='/'.join(__file__.split('/')[:-1])
        font = ImageFont.truetype(filefolder+"/impact.ttf", fontSize)
        w, h = draw.textsize(msg, font)

        imgWidthWithPadding = img.width * 0.99

        # 1. how many lines for the msg to fit ?
        lineCount = 1
        if(w > imgWidthWithPadding):
            lineCount = int(round((w / imgWidthWithPadding) + 1))

        if lineCount > 2:
            while 1:
                fontSize -= 2
                font = ImageFont.truetype(filefolder+"/impact.ttf", fontSize)
                w, h = draw.textsize(msg, font)
                lineCount = int(round((w / imgWidthWithPadding) + 1))
                # print("try again with fontSize={} => {}".format(fontSize, lineCount))
                if lineCount < 3 or fontSize < 10:
                    break


        # print("img.width: {}, text width: {}".format(img.width, w))
        # print("Text length: {}".format(len(msg)))
        # print("Lines: {}".format(lineCount))


        # 2. divide text in X lines
        lastCut = 0
        isLast = False
        for i in range(0,lineCount):
            if lastCut == 0:
                cut = (len(msg) / lineCount) * i
            else:
                cut = lastCut
            cut=int(cut) # have to be integer
            if i < lineCount-1:
                nextCut = (len(msg) / lineCount) * (i+1)
            else:
                nextCut = len(msg)
                isLast = True
            nextCut=int(nextCut)
            # print("cut: {} -> {}".format(cut, nextCut))

            # make sure we don't cut words in half
            if nextCut == len(msg) or msg[nextCut] == " ":
                pass
            else:
                while msg[nextCut] != " " and nextCut != len(msg)-1: ## index out of range in long words?
                    nextCut += 1
            line = msg[cut:nextCut].strip()

            # is line still fitting ?
            w, h = draw.textsize(line, font)
            if not isLast and w > imgWidthWithPadding:
                # print("overshot")
                nextCut -= 1
                while msg[nextCut] != " ":
                    nextCut -= 1
                # print("new cut: {}".format(nextCut))

            lastCut = nextCut
            lines.append(msg[cut:nextCut].strip())

        # print(lines)

        # 3. print each line centered
        lastY = -h
        if pos == "bottom":
            lastY = img.height - h * (lineCount+1) - 10

        for i in range(0,lineCount):
            w, h = draw.textsize(lines[i], font)
            textX = img.width/2 - w/2
            #if pos == "top":
            #    textY = h * i
            #else:
            #    textY = img.height - h * i
            textY = lastY + h
            draw.text((textX-2, textY-2),lines[i],(0,0,0),font=font)
            draw.text((textX+2, textY-2),lines[i],(0,0,0),font=font)
            draw.text((textX+2, textY+2),lines[i],(0,0,0),font=font)
            draw.text((textX-2, textY+2),lines[i],(0,0,0),font=font)
            draw.text((textX, textY),lines[i],(255,255,255),font=font)
            lastY = textY
        return




from PIL import Image
from PIL import ImageFont
from PIL import ImageDraw

img = Image.open("images/one-does-not-simply.jpg")
draw = ImageDraw.Draw(img)
font = ImageFont.truetype("impact.ttf", 52)

draw.text((10, 10), "Hello world", (255,255,255), font=font)
img.save("out.jpg")



import requests as req
from os.path import join
from random import randint
import sys

class PixaBay(object):
    '''
        PixaBay - python wrapper to query the pixabay website
        
        ``desc``:
                takes in an api key from pixabay and allows you
                to search for photos
        ``param``:
                key<str> - the api key used by the developer
                visit https://pixabay.com/api/ to get one, it's free
        ``Method``:
            get_images(self,search, **) - query pixabay and return all images
            matching your search
    '''
    def __init__(self, key):
        
        self.base_url = 'https://pixabay.com/api/'
        self.key = key
        
    def get_images(self, search,per_page=20,image_type='photo',pretty='false',category='backgrounds',minWidth=64,minHeight=64,orientation='horizontal',safesearch='true',page=1,order='popular',lang='en',editors_choice='false'):
        '''
            get_images(self, search,per_page=20,image_type='photo',pretty='false',category='backgrounds',minWidth=64,minHeight=64,orientation='horizontal',safesearch='true',page=1,order='popular',lang='en',editors_choice='false') --> PixaCollection

            ``param``:
                    search<str> - your search terms e.g 'woman using laptop'
            ``param``:
                    The rest of the params are not required but if you'd
                    like to know more about them visit https://pixabay.com/api/

            ``Example``:
                    images = bay.get_images(search='tiger hd background')#returns a PixaCollection instance
                    images.download_hits('./')#will download all the image results 
        '''
        search = search.replace(' ','+')
        url = '?'.join([self.base_url,'key=%s&q=%s&image_type=%s&pretty=%s&category=%s&minWidth=%s&minHeight=%s&orientation=%s&safesearch=%s&order=%s&page=%s&per_page=%s&lang=%s&editors_choice=%s'%(self.key,search,image_type,pretty,category,minWidth,minHeight,orientation,safesearch,order,page,per_page,lang,editors_choice)])

        r = req.get(url).json()
        import pickle
        with open('test.pb', 'wb') as f:
            pickle.dump(r, f)

        return PixaCollection(r)
    
class PixaCollection(object):
    '''
        PixaCollection - the class returned by `PixaBay`` class.
                    One should never have to use this class explicitly
                    -The class will contain all data you need to work
                    with the pixabay results as well as some utility functions
                    to help you do popular tasks easily
        ``param``:
                image_data - data returned by PixaBay().get_images()
        
        ``Methods``:
            download_hits(self, path) - download all the images
            
            download_random(self, path) - download a single image at random
            
            get_img(self, img_no) - return all information pertaining 
            to a specific image
    '''
    def __init__(self, img_data):
        self.img_data = img_data

        self.total_hits = self.img_data['totalHits']
        self.total = self.img_data['total']
        self.hits = self.img_data['hits']

    def download_hits(self, path):
        '''
            download_hits(self, path)
            download all the images returned by the PixaBay.get_images() function

            ``param``:
                    path<str> - the path where the images will be saved
            ``example``:
                    images = bay.get_images()
                    images.download_hits('path/to/save_folder')
        '''
        
        for i,hit in enumerate(self.hits):
            tgt_img = hit['largeImageURL']
            ext = tgt_img[tgt_img.rfind('.'):]
            tags = hit['tags'].split(',')
            name = '_'.join([hit['user'], tags[0], str(i)])+ext
            image = req.get(tgt_img).content

            with open(join(path,name), 'wb') as img_base:
                img_base.write(image)


    def download_random(self, path):
        '''
            download_random(self, path)

            download a random image from the returned results
            Downloads only one image from the list, useful for testing or
            when you just need a quick image to work with

            ``param``:
                path<str> - the path to save the image to
            ``example``:
                    images = bay.get_images()
                    images.download_random('path/to/save_folder')
        '''
        tgt_idx = randint(0,len(self.hits))
        hit = self.hits[tgt_idx]

        tgt_img = hit['largeImageURL']
        ext = tgt_img[tgt_img.rfind('.'):]
        tags = hit['tags'].split(',')
        name = '_'.join([hit['user'], tags[0], str(i)])+ext
        image = req.get(tgt_img).content

        with open(join(path,name), 'wb') as img_base:
            img_base.write(image)

    def get_img(self, img_no):
        '''
            get_img(self, img_no) --> PixaImage

            get an image from the api call results and
            return all the info for that image.

            ``param``:
                img_no<int> - the index of the image to get
                Should be between 0 - len(returned_results) 
        '''
        try:
            tgt_img = self.hits[img_no]
        except Exception as e:
            print('Image Index Should Be Within 0 - %s: \n%s'%(len(self.hits),e))
            sys.exit()
        #print('tggt',tgt_img) 
        return PixaImage(tgt_img)

class PixaImage(object):
    '''
        PixaImage - the class contains all the data of an image
        exposed by the api

        ``param``:
            img_data<str> - all the data of the image 
            as returned by PixaCollection.get_img()

        ``Methods``:
            get_img_attr(self, attr='id') - returns an attribute of an image 

            download(self, path, size='default') - download this image   
    '''
    def __init__(self, img_data):

        self.img_data = img_data

    def get_img_attr(self, attr='id'):
        '''
            get_img_attr(self, attr='id')

            Get an image's attribute provided by the pixabay api
            Make sure you query an attribute that exists in the api website
            
            ``example``:
                uploaded_by = get_img_attr('user')#returns the photographer
        '''
        try:
            return self.img_data[attr]
        except Exception as e:
            print("Attribute Doesn't Exist, Should be one of:")
            for k in self.img_data.keys():
                print(k)
            print(e)
    
    def download(self, path, name='',size='default'):
        '''
            download(self, path, size='default')

            download the image to tyour disk

            ``param``:
                path<str> - the path to save the file to
            ``param``:
                size<size> - the size to download the image in.
                Should be one of default,preview,web,large
        '''
        if size == 'large':
            s = 'largeImageURL'
        elif size == 'preview':
            s = 'previewURL'
        elif size == 'web':
            s = 'webformatURL'
        elif size == 'default':
            s = 'imageURL'
        else:
            sys.exit('Invalid Size Option, should be one of:\n largeImageURL, previewURL, webformatURL, imageURL')
        
        tgt_img = self.img_data[s]
        ext = tgt_img[tgt_img.rfind('.'):]
        tags = self.img_data['tags'].split(',')
        if name == '':
            name = '_'.join([self.img_data['user'], tags[0]])+ext
        else:
            name=name.split('.')[0] ## override any extension given
            name=name+ext

        image = req.get(tgt_img).content

        with open(join(path,name), 'wb') as img_base:
            img_base.write(image)

        return name ## return image name



import genanki
from LangDeckGen.AnkiCard import AnkiCard
from LangDeckGen import WordList
from LangDeckGen.AnkiDeck import AnkiDeck
from LangDeckGen.AnkiModel import AnkiModel
from time import sleep
import logging
import shutil
class LangDeck:
    def __init__(self, deck_name: str, wordlist: WordList, **kwargs):
        self.deck_name = deck_name
        self.wordlist=wordlist
        self.deck=self.createLangDeck(self.deck_name,self.wordlist,**kwargs)
        self.package=self.packageLangDeck(self.deck,**kwargs)
        self.outputLangDeck(self.package)
        self.clearMedia()

    def createLangDeck(self, deck_name, wordlist, **kwargs) -> AnkiDeck:
        def chunk_list (mylist,x):
            return [mylist[i:i+x] for i in range(0, len(mylist), x)]
        deck_name = deck_name.replace(" ","-")
        anki_model = AnkiModel(**kwargs)
        anki_cards = list()
        chunk_size=kwargs.get('chunk_size',10)
        for chunk in chunk_list(wordlist.translation_list,chunk_size):
            ## this loop inserts entries, chunk_size entries at a time
            for entry in chunk:
                word,lang=map(str.strip,entry[:2])
                args=list(map(str.strip,entry[2:]))
                try:
                    anki_card=AnkiCard(str(word),str(lang),*args,**kwargs)
                    anki_cards.append(anki_card)
                except Exception as e:
                    print(e)
                    print(f"Coundnt generate card for {word}.")
                sleep(2)
                logging.info(f"[ {word} ] card added. Sleeping for 2 seconds.")
            ## this part will output a langdeck, it increments at every 10.
            deck = AnkiDeck(title=deck_name, anki_cards=anki_cards)
            deck_notes = deck.create_notes(anki_model,**kwargs)
            for note in deck_notes:
                deck.add_note(note)
            logging.info("Waiting 60 s to start next chunk of cards...")
            sleep(60) ## sleep so site doesnt complain
            package=self.packageLangDeck(deck,**kwargs)
            self.outputLangDeck(package) ## generates partial deck every 10 new entries
        #self.clearMedia()
        return deck

    def packageLangDeck(self,deck,**kwargs):
        def package_deck(new_deck: genanki.Deck, media: list) -> genanki.Package:
            TL_package = genanki.Package(new_deck)
            TL_package.media_files = media
            return TL_package
        package = deck.package_deck(deck,**kwargs)
        return package

    def outputLangDeck(self,package):
        package.write_to_file(f"{self.deck_name}.apkg")

    def clearMedia(self):
        shutil.rmtree("./tmp/imgs")
        shutil.rmtree("./tmp/sound")



import csv
import errno
import logging
import os
import shutil
import sys
from pathlib import Path
from time import sleep

#import genanki

from LangDeckGen.AnkiCard import AnkiCard
from LangDeckGen.AnkiDeck import AnkiDeck
from LangDeckGen.AnkiModel import AnkiModel
#from LangDeckGen.WordList import WordList
import wget 
class WordList:
    def __init__(self, lang, index):
        self.lang = lang
        if index == 'automatic':
            self.i_idx = 0
            self.f_idx = None
        if isinstance(index,int):
            print(f'Length of WordList specified: {index}.')
            self.f_idx = index
            self.i_idx = 0
            self.length = index
        elif isinstance(index,list):
            self.i_idx, self.f_idx = index
            self.length = self.f_idx - self.i_idx
            print(f'Wordlists starts and end at: {self.i_idx} {self.f_idx}')
        self.rawlist = ""
        self.translation_list = []

    def __str__(self):
        return f"WordList(Language: {self.lang},\n Length: {self.length},\n List: {self.translation_list})"

    def downloadWordList(self):
        self.rawlist=self.__download_raw_word_list()
#        self.__process_list(self.rawlist)

    def __download_raw_word_list(self):
        url = f"https://raw.githubusercontent.com/hermitdave/FrequencyWords/master/content/2018/{self.lang}/{self.lang}_50k.txt"
        filename = wget.download(url)
        newtxt=""
        with open(filename, "r") as f:
            lines=f.readlines()
            for line in lines:
                newtxt+=line.split()[0]+f"; {self.lang}; 1\n"
        with open(filename,"w") as f:
            f.write(newtxt)
        return filename

    def __process_list(self, filename):
        newtxt=""
        with open(filename, "r") as f:
            lines=f.readlines()
            for line in lines:
                if len(line.split(';')) <= 2: ## so dont do this over again
                    newtxt+=line.strip()+f"; {self.lang}; 1\n"
                else:
                    newtxt+=line.strip()+"\n"
        with open(filename,"w") as f:
            f.write(newtxt)

    def importWordList(self, csv_file="", rawList=True) -> list:
        if csv_file == "": ## if nothing specified gets default from lang
            csv_file=f"{self.lang}_50k.txt"

        if rawList: ## case it just contains the phrases line by line
            self.__process_list(csv_file) ## it will read the file and add ; {selflang}

        translation_list = list()
        try:
            with Path(csv_file).open(encoding="utf-8") as stream:
                reader = csv.reader(stream,delimiter=';')
                for row in reader:
                    translation_list.append(row)
        except FileNotFoundError:
            logging.critical(f"Unable to locate '{csv_file}'. Check it exists.")
            sys.exit(errno.EIO)
        except TypeError:
            logging.critical(f"Unable to load '{csv_file}'. Check it is populated.")
            sys.exit(errno.EIO)
        else:
            self.translation_list=translation_list[self.i_idx:self.f_idx] ## returns list up to length specified
    
    def filterWordList(self, entries_to_filter):
        new_translation_list = []
        for sublist in self.translation_list:
            if sublist[0] not in entries_to_filter:
                if sublist[0] not in new_translation_list:
                    new_translation_list.append(sublist)
        self.translation_list=new_translation_list



from PIL import Image
from PIL import ImageFont
from PIL import ImageDraw

img = Image.open("image.jpg")
draw = ImageDraw.Draw(img)
font = ImageFont.truetype("impact.ttf", 42)

def drawTextWithOutline(text, x, y):
    draw.text((x-2, y-2), text,(0,0,0),font=font)
    draw.text((x+2, y-2), text,(0,0,0),font=font)
    draw.text((x+2, y+2), text,(0,0,0),font=font)
    draw.text((x-2, y+2), text,(0,0,0),font=font)
    draw.text((x, y), text, (255,255,255), font=font)
    return

text = "One does not simply walk into mordor".upper()
w, h = draw.textsize(text, font) # measure the size the text will take

if w > img.width:
    lineCount = int(round((w / img.width) + 1))

# print("lineCount: {}".format(lineCount))

lines = []
if lineCount > 1:

    lastCut = 0
    isLast = False
    for i in range(0,lineCount):
        if lastCut == 0:
            cut = (len(text) / lineCount) * i
        else:
            cut = lastCut

        if i < lineCount-1:
            nextCut = (len(text) / lineCount) * (i+1)
        else:
            nextCut = len(text)
            isLast = True

        # print("cut: {} -> {}".format(cut, nextCut))

        # make sure we don't cut words in half
        if nextCut == len(text) or text[nextCut] == " ":
            # print("may cut")
        else:
            # print("may not cut")
            while text[nextCut] != " ":
                nextCut += 1
            # print("new cut: {}".format(nextCut))

        line = text[cut:nextCut].strip()

        # is line still fitting ?
        w, h = draw.textsize(line, font)
        if not isLast and w > img.width:
            # print("overshot")
            nextCut -= 1
            while text[nextCut] != " ":
                nextCut -= 1
            # print("new cut: {}".format(nextCut))

        lastCut = nextCut
        lines.append(text[cut:nextCut].strip())

else:
    lines.append(text)

# print(lines)

for i in range(0, lineCount):
    w, h = draw.textsize(lines[i], font)
    drawTextWithOutline(lines[i], img.width/2 - w/2, i * h)

img.save("out.jpg")



import wave
import sys
from pydub import AudioSegment
import soundfile as sf
import pyrubberband as pyrb
## based on code from: https://stackoverflow.com/users/2866150/lokesh-deshmukh

def ChangeAudioSpeed(mp3file : str, speed : float):
    """ this will rewrite mp3 file with update speed """
    sound = AudioSegment.from_mp3(mp3file)
    sound.export("tmp.wav", format="wav")
    y, sr = sf.read("tmp.wav")
    # Play back at extra low speed
    y_stretch = pyrb.time_stretch(y, sr, speed)
    # Play back extra low tones
    y_shift = pyrb.pitch_shift(y, sr, speed)
    sf.write("tmp_stretched.wav", y_stretch, sr, format='wav')

    sound = AudioSegment.from_wav("tmp_stretched.wav")
    mp3file=mp3file.split('.')[0]+"-"+str(speed)+".mp3"
    sound.export(mp3file, format="mp3")
    return mp3file




from pathlib import Path

import genanki
import LangDeckGen
from LangDeckGen import AnkiTemplates
import os
import random

class AnkiModel(genanki.Model):
    try: 
        os.mkdir("./tmp")
    except Exception as e:
        print(e)
    try: 
        os.mkdir("./tmp/sound")
    except Exception as e:
        print(e)
    try: 
        os.mkdir("./tmp/imgs")
    except Exception as e:
        print(e)
    model_id = str
    fields = [
        {"name": "Word_TL"},
        {"name": "Word_EN"},
        {"name": "Audio_Word"},
        {"name": "Phr1_TL"},
        {"name": "Phr1_EN"},
        {"name": "Audio_P1"},
        {"name": "Phr2_TL"},
        {"name": "Phr2_EN"},
        {"name": "Audio_P2"},
       # {"name": "Phr3_TL"},
       # {"name": "Phr3_EN"},
       # {"name": "Audio_P3"},
        {"name": "Image_Word"},
        {"name": "Dict_Entry"},
#        {"name": "Phr4_TL"},
#        {"name": "Phr4_EN"},
#        {"name": "Audio_S4"},
    ]
    fields_phrs = [
        {"name": "Word_TL"},
        {"name": "Word_EN"},
        {"name": "Audio_Word"},
        {"name": "Image_Word"},
    ]
    templates = AnkiTemplates.MODEL_TEMPLATES
    templates_phr = AnkiTemplates.MODEL_TEMPLATES_PHR
    packagefolder=Path(LangDeckGen.__file__).parent
    css_file= packagefolder / Path("templates/anki.css")
    css = css_file.open().read()

    def __init__(self, **kwargs):
        if not kwargs.get("PhrasesOnly"):
            super(AnkiModel, self).__init__(name="Simple Vocabulary",
                                            model_id=self.__create_id("Simple Vocabulary"), 
                                            fields=self.fields,
                                            templates=self.templates,
                                            css=self.css)
        else:
            super(AnkiModel, self).__init__(name="Simple Vocabulary2",
                                            model_id=self.__create_id("Simple Vocabulary2"), 
                                            fields=self.fields_phrs,
                                            templates=self.templates_phr,
                                            css=self.css)

    def __create_id(self,name : str):
        id=0
        for char in name:
            id+=ord(char)
        return id


    def __str__(self):
        return f"<AnkiModel {self.model_id}>"




from PIL import Image
from PIL import ImageFont
from PIL import ImageDraw

img = Image.open("image.jpg")
draw = ImageDraw.Draw(img)
font = ImageFont.truetype("impact.ttf", 52)

textX = 10
textY = 10
text = "Hello outline"

draw.text((textX-2, textY-2), text,(0,0,0),font=font)
draw.text((textX+2, textY-2), text,(0,0,0),font=font)
draw.text((textX+2, textY+2), text,(0,0,0),font=font)
draw.text((textX-2, textY+2), text,(0,0,0),font=font)
draw.text((textX, textY), text, (255,255,255), font=font)
img.save("out.jpg")



import os, codecs
from setuptools import setup, find_packages
here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.0.8'
DESCRIPTION = 'A package to produce Anki decks from list of words for language learning including automatic audio and image generation.'

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

# Setting up
setup(
    name="LangDeckGen",
    version=VERSION,
    author="Rogerio Gouvea",
    author_email="<rogeriog.em@gmail.com>",
    description=DESCRIPTION,
    packages=find_packages() + ['LangDeckGen','LangDeckGen/templates/'],
    install_requires=requirements,
    include_package_data=True,
    long_description_content_type="text/markdown",
    long_description=long_description,
    keywords=['python', 'tts', 'anki', 'flashcards', 'language'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
    ]
)




MODEL_TEMPLATES = [
    {
        "name": "TL to English",
        # Front template format
        "qfmt": """
{{Word_TL}}{{Audio_Word}}<br>
{{Image_Word}}
        """,
        # Back template Format
        "afmt": """
{{Word_TL}}
{{Audio_Word}}


<hr id=answer>

{{Word_EN}}

<hr>

{{#Phr1_TL}}
<div style='font-family: Arial; font-size: 16px;'>{{Phr1_TL}}{{Audio_P1}}</div>
<div style='font-family: Arial; font-size: 14px;'>{{hint:Phr1_EN}}</div><br>
{{/Phr1_TL}}

{{#Phr2_TL}}
<div style='font-family: Arial; font-size: 16px;'>{{Phr2_TL}}{{Audio_P2}}</div>
<div style='font-family: Arial; font-size: 14px;'>{{hint:Phr2_EN}}</div><br>
{{/Phr2_TL}}

{{#Dict_Entry}}
<div style='font-family: Arial; font-size: 14px;'>{{hint:Dict_Entry}}</div><br>
{{/Dict_Entry}}
        """
    },]
MODEL_TEMPLATES_PHR = [
    {
        "name": "TL to English - Phrases",
        # Front template format
        "qfmt": """
{{Word_TL}}{{Audio_Word}}<br>
{{Image_Word}}
        """,
        # Back template Format
        "afmt": """
{{Word_TL}}
{{Audio_Word}}


<hr id=answer>

{{Word_EN}}

        """
    },
]
#{{#Satz4_TL}}
#<div style='font-family: Arial; font-size: 16px;'>{{Satz4_TL}}{{Audio_P4}}</div>
#<div style='font-family: Arial; font-size: 14px;'>{{hint:Satz4_EN}}</div><br>
#{{/Satz4_TL}}
